@@ -1,4 +1,4 @@
-from qaseio.pytest.plugin import QasePytestPluginSingleton
+from qaseio.pytest.plugin import QasePytestPlugin, QasePytestPluginSingleton
 
 
 def get_option_ini(config, name):
@@ -56,6 +56,14 @@ def pytest_addoption(parser):
         help="Create new testrun, if no testrun id provided",
         action="store_true",
     )
+    add_option_ini(
+        "--qase-complete-run",
+        "qs_complete_run",
+        default=False,
+        type="bool",
+        help="Complete run after all tests are finished",
+        action="store_true",
+    )
     add_option_ini(
         "--qase-debug",
         "qs_debug",
@@ -67,19 +75,30 @@ def pytest_addoption(parser):
 
 
 def pytest_configure(config):
-    config.addinivalue_line(
-        "markers", "qase(*ids): mark test to be associate with Qase TMS"
-    )
-    if get_option_ini(config, "qs_enabled"):
-        QasePytestPluginSingleton.init(
-            api_token=get_option_ini(config, "qs_api_token"),
-            project=get_option_ini(config, "qs_project_code"),
-            testrun=get_option_ini(config, "qs_testrun_id"),
-            testplan=get_option_ini(config, "qs_testplan_id"),
-            create_run=get_option_ini(config, "qs_new_run"),
-            debug=get_option_ini(config, "qs_debug"),
-        )
-        config.pluginmanager.register(
-            QasePytestPluginSingleton.get_instance(),
-            name="qase-pytest",
+    if not hasattr(config, "slaveinput"):
+        QasePytestPlugin.drop_run_id()
+        config.addinivalue_line(
+            "markers", "qase(*ids): mark test to be associate with Qase TMS"
         )
+        if get_option_ini(config, "qs_enabled"):
+            QasePytestPluginSingleton.init(
+                api_token=get_option_ini(config, "qs_api_token"),
+                project=get_option_ini(config, "qs_project_code"),
+                testrun=get_option_ini(config, "qs_testrun_id"),
+                testplan=get_option_ini(config, "qs_testplan_id"),
+                create_run=get_option_ini(config, "qs_new_run"),
+                complete_run=get_option_ini(config, "qs_complete_run"),
+                debug=get_option_ini(config, "qs_debug"),
+            )
+            config.qaseio = QasePytestPluginSingleton.get_instance()
+            config.pluginmanager.register(
+                config.qaseio,
+                name="qase-pytest",
+            )
+
+
+def pytest_unconfigure(config):
+    qaseio = getattr(config, "qaseio", None)
+    if qaseio:
+        del config.qaseio
+        config.pluginmanager.unregister(qaseio)
