@@ -1,4 +1,5 @@
 import logging
+import pathlib
 import time
 from datetime import datetime
 from typing import Tuple, Union
@@ -6,6 +7,7 @@ from typing import Tuple, Union
 import pytest
 
 import apitist
+from filelock import FileLock
 
 from qaseio.client import QaseApi
 from qaseio.client.models import (
@@ -25,6 +27,13 @@ PYTEST_TO_QASE_STATUS = {
     "SKIPPED": TestRunResultStatus.BLOCKED,
 }
 
+try:
+    from xdist import is_xdist_master
+except ImportError:
+
+    def is_xdist_master(*args, **kwargs):
+        return True
+
 
 def get_ids_from_pytest_nodes(items):
     """Return tuple with item and test case ids for it and tests missing ids"""
@@ -50,6 +59,7 @@ class MissingStepIdentifierException(Exception):
 
 class QasePytestPlugin:
     testrun: TestRunInfo = None
+    meta_run_file = pathlib.Path("qaseio.runid")
 
     def __init__(
         self,
@@ -58,6 +68,7 @@ class QasePytestPlugin:
         testrun=None,
         testplan=None,
         create_run=False,
+        complete_run=False,
         debug=False,
     ):
         self.client = QaseApi(api_token)
@@ -65,6 +76,7 @@ class QasePytestPlugin:
         self.testrun_id = testrun
         self.testplan_id = testplan
         self.create_run = create_run
+        self.complete_run = complete_run
         self.debug = debug
         self.nodes_with_ids = {}
         self.cases_info = {}
@@ -133,7 +145,7 @@ class QasePytestPlugin:
             )
 
     def load_testrun(self):
-        if self.testrun_id:
+        if self.testrun_id and self.testrun is None:
             self.testrun = self.client.runs.exists(
                 self.project_code,
                 self.testrun_id,
@@ -204,39 +216,46 @@ class QasePytestPlugin:
 
         Prints additional info at start of the run, if debug in True
         """
-        self.nodes_with_ids, no_ids = get_ids_from_pytest_nodes(items)
-        write_lines = []
-        if no_ids:
-            line = ", ".join([miss.name for miss in no_ids])
-            write_lines.append(
-                "This tests does not have test case ids:\n" + line
-            )
-
-        for nodeid, data in self.nodes_with_ids.items():
-            exist_ids, missing_ids = self.check_case_ids(data)
-            if missing_ids:
+        with FileLock("qaseio.lock"):
+            self.load_run_from_lock()
+            self.load_testrun()
+            self.nodes_with_ids, no_ids = get_ids_from_pytest_nodes(items)
+            write_lines = []
+            if no_ids:
+                line = ", ".join([miss.name for miss in no_ids])
                 write_lines.append(
-                    "For test {} could not find test cases in TMS: {}".format(
-                        nodeid, ", ".join([str(i) for i in missing_ids])
-                    )
+                    "This tests does not have test case ids:\n" + line
                 )
 
-            if not self.create_run:
-                missing_in_run = self.get_missing_in_testrun(data)
-                if missing_in_run:
+            for nodeid, data in self.nodes_with_ids.items():
+                exist_ids, missing_ids = self.check_case_ids(data)
+                if missing_ids:
                     write_lines.append(
-                        "For test {} could not find "
-                        "test cases in run: {}".format(
-                            nodeid, ", ".join([str(i) for i in missing_in_run])
+                        "For test {} can't find test cases in TMS: {}".format(
+                            nodeid, ", ".join([str(i) for i in missing_ids])
                         )
                     )
-                self.missing_ids.extend(missing_in_run)
-            self.missing_ids.extend(missing_ids)
-            self.existing_ids.extend(exist_ids)
 
-        if self.create_run and self.existing_ids:
-            self.create_testrun(self.existing_ids)
-            self.load_testrun()
+                if not self.create_run:
+                    missing_in_run = self.get_missing_in_testrun(data)
+                    if missing_in_run:
+                        write_lines.append(
+                            "For test {} could not find "
+                            "test cases in run: {}".format(
+                                nodeid,
+                                ", ".join([str(i) for i in missing_in_run]),
+                            )
+                        )
+                    self.missing_ids.extend(missing_in_run)
+                self.missing_ids.extend(missing_ids)
+                self.existing_ids.extend(exist_ids)
+
+            if self.create_run and self.existing_ids and not self.testrun_id:
+                self.create_testrun(self.existing_ids)
+                self.load_testrun()
+
+                with open(self.meta_run_file, "w") as lock_file:
+                    lock_file.write(str(self.testrun_id))
 
         if write_lines and self.debug:
             writer = config.pluginmanager.get_plugin("terminalreporter")
@@ -247,6 +266,30 @@ class QasePytestPlugin:
                 writer.line(line)
             writer.section("Qase TMS setup finished", sep="=")
 
+    def load_run_from_lock(self):
+        if self.meta_run_file.exists():
+            with open(self.meta_run_file, "r") as lock_file:
+                try:
+                    self.testrun_id = int(lock_file.read())
+                except ValueError:
+                    pass
+
+    @staticmethod
+    def drop_run_id():
+        if QasePytestPlugin.meta_run_file.exists():
+            QasePytestPlugin.meta_run_file.unlink()
+
+    def pytest_sessionstart(self, session):
+        if is_xdist_master(session):
+            QasePytestPlugin.drop_run_id()
+
+    def pytest_sessionfinish(self, session, exitstatus):
+        if is_xdist_master(session):
+            if self.complete_run:
+                self.load_run_from_lock()
+                self.complete()
+            QasePytestPlugin.drop_run_id()
+
     @pytest.hookimpl(hookwrapper=True)
     def pytest_runtest_protocol(self, item):
         self.start_pytest_item(item)
@@ -342,20 +385,35 @@ class QasePytestPlugin:
             attachments.extend(files)
             node["attachments"] = attachments
 
+    def complete(self):
+        if self.testrun_id and self.complete_run:
+            print()
+            print(f"Finishing run {self.testrun_id}")
+            res = self.client.runs.get(self.project_code, self.testrun_id)
+            if res.status == 1:
+                print(f"Run {self.testrun_id} already finished")
+                return
+            try:
+                self.client.runs.complete(self.project_code, self.testrun_id)
+                print(f"Run {self.testrun_id} was finished successfully")
+            except Exception as e:
+                print(f"Run {self.testrun_id} was finished with error: {e}")
+
 
 class QasePytestPluginSingleton:
-    __instance = None
+    _instance = None
 
     @staticmethod
     def init(**kwargs):
-        QasePytestPluginSingleton.__instance = QasePytestPlugin(**kwargs)
+        if QasePytestPluginSingleton._instance is None:
+            QasePytestPluginSingleton._instance = QasePytestPlugin(**kwargs)
 
     @staticmethod
     def get_instance() -> QasePytestPlugin:
         """ Static access method"""
-        if QasePytestPluginSingleton.__instance is None:
+        if QasePytestPluginSingleton._instance is None:
             raise Exception("Init plugin first")
-        return QasePytestPluginSingleton.__instance
+        return QasePytestPluginSingleton._instance
 
     def __init__(self):
         """ Virtually private constructor"""
